
/* --- C ---
************************************************************************
*
*   Filename    : mpegjoin.c
*   Description : joins two or more MPEG-1-Video-streams
*   Part of     : MPEGJOIN
*
*   Version     : 1.1
*   Date        : Sat Dec  9 16:24:19 EST 1995
*   Language    : C
*   For machine : Linux, Sun OS, SCO, AU/X, Solaris
*   Compile as  : see Makefile
*
*   Author      : Frank Gadegast
*   Contact     : phade@cs.tu-berlin.de
*   Copyright   : PHADE Software, Berlin
*
************************************************************************
*/

#include <stdio.h>
#include <fcntl.h>
#include <sys/stat.h>

#ifdef DOS
#define INPUT_ACCESS  O_BINARY|O_RDONLY
#define OUTPUT_ACCESS O_BINARY|O_CREAT|O_TRUNC|O_RDWR
#else
#define INPUT_ACCESS  O_RDONLY
#define OUTPUT_ACCESS O_CREAT|O_TRUNC|O_RDWR
#endif
#define OUTPUT_PERMS  S_IWRITE|S_IREAD

#define BUFSIZE 10000

char buf[BUFSIZE];

#include <sys/timeb.h>

void share_wait ()
{
  struct timeb    t1,t2;
  int j;

  ftime(&t1);
  do
  {
    for (j=0; j<500; j++)
    ftime(&t2);
    fprintf (stderr, ".");
  }
  while ((t2.time - t1.time) < 10);
  fprintf (stderr, "\n");
}

int main(argc, argv)
int argc;
char *argv[];
{
	int f, s, n;
	int c, i, j, sk=0;
	int arg;


#ifdef SHARE
	fprintf (stderr, "\nMPEGJOIN Version 1.1, 07/12/95, MPEG-1-Video Joiner\n");
	fprintf (stderr, "         Copyright (C) 1994-95 by PHADE Software\n\n");

	fprintf(stderr, "Non-registered shareware-version. Please read the\n");
	fprintf(stderr, "README.TXT-file to support shareware.\n\n");
#else
	printf ("\nMPEGJOIN Version 1.1, 07/12/95, MPEG-1-Video Joiner\n");
	printf ("         Copyright (C) 1994-95 by PHADE Software\n\n");
#endif

	if (argc < 4)
	{
		fprintf (stderr, "usage: %s new.mpg first.mpg second.mpg ...\n", argv[0]);
		exit (-1);
	}

	n = open (argv[1], OUTPUT_ACCESS, OUTPUT_PERMS);
	if (n == 0)
	{
		fprintf (stderr, "%s: can't open output stream '%s'\n", argv[0], argv[3]);
		exit (-1);
	}

	arg = 2;
	while (arg < argc)
	{
		s = open (argv[arg], INPUT_ACCESS);
		if (s == 0)
		{
			fprintf (stderr, "%s: can't open input stream '%s'\n", argv[0], argv[arg]);
			exit (-1);
		}
		printf ("Adding stream '%s' ...\n", argv[arg]);
#ifdef SHARE
	share_wait ();
#endif
	
		sk = 0;
		if (arg > 2)
		{
			c = read (s, buf, BUFSIZE);
			for (i=0; i<c; i++)
				if ((buf[i] == 0) &&
					(buf[i+1] == 0) &&
					(buf[i+2] == 1) &&
					(buf[i+3] == -72))
				{
					sk = i;
					break;
				}
			if (sk != c) printf ("Skipping %d bytes for unused sequence header ...\n", sk);
			write (n, buf+sk, c-sk);
		}
		sk = 0;
		do
		{
			c = read (s, buf, BUFSIZE);
			i = c;
			if ((c != BUFSIZE) && (arg < argc-1))
			{
				do
				{
					if ((buf[i] == 0) &&
						(buf[i+1] == 0) &&
						(buf[i+2] == 1) &&
						(buf[i+3] == -73))
					{
						sk = c-i;
						c = i;
						break;
					}
					i--;
				} while (i > 3);
			}
			write (n, buf, c);
		} while (c == BUFSIZE);
		if (sk != 0) printf ("Skipping %d bytes for unused sequence ending code ...\n", sk);
		close (s);
		arg++;
	}
	close (n);

	exit (0);
}

